function op = opWavelet(m,n, family, filter, levels, type)
% OPWAVELET  Wavelet operator
%
%    OPWAVELET(M,N,FAMILY,FILTER,LEVELS,TYPE) creates a wavelet
%    operator of given FAMILY, for M by N matrices. The wavelet
%    transformation is computed using the Rice Wavelet Toolbox.
%
%    The remaining three parameters are optional. FILTER = 8
%    specifies the filter length and must be even. LEVELS = 5 gives
%    the number of levels in the transformation. Both M and N must
%    be divisible by 2^LEVELS. TYPE = 'min' indictates what type of
%    solution is desired; 'min' for minimum phase, 'max' for
%    maximum phase, and 'mid' for mid-phase solutions. 
%
%   Url: http://unlocbox.sourceforge.net/rr/image_source_separation/misc/opWavelet.php

% Copyright (C) 2013 LTS2-EPFL, by Nathanael Perraudin.
% This file is part of UNLocBoX-RR version 0.1.10
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

%   Copyright 2007, Rayan Saab, Ewout van den Berg and Michael P. Friedlander
%   http://www.cs.ubc.ca/labs/scl/sparco
%   $Id: opWavelet.m 577 2007-09-17 01:48:28Z mpf $

if (nargin < 3), family = 'Daubechies';    end;
if (nargin < 4), filter =     8;           end;
if (nargin < 5), type   = 'min';           end;
if (nargin < 6), levels = floor(log2(n));  end;

family = lower(family);

switch family
 case {'daubechies'}
    h = daubcqf(filter);
    
 case {'haar'}
    h = daubcqf(0);
    
 otherwise
    error(sprintf('Wavelet family %s is unknown', family));
end

op = @(x,mode) opWavelet_intrnl(m,n,family,filter,levels,type,h,x,mode);


function y = opWavelet_intrnl(m,n,family,filter,levels,type,h,x,mode)
if mode == 0
   y = {n*m,n*m,[0,1,0,1],{'Wavelet',family,filter,levels,type}};
elseif mode == 1
   if isreal(x)
     [y,l] = midwt(reshape(x,[m,n]), h, levels);
   else
     [y1,l] = midwt(reshape(real(x),[m,n]), h, levels);
     [y2,l] = midwt(reshape(imag(x),[m,n]), h, levels);
     y = y1 + sqrt(-1) * y2;    
   end
   y = reshape(y,[m*n,1]);
else
   if isreal(x)
      [y,l] = mdwt(reshape(x,[m,n]), h, levels);
   else
      [y1,l] = mdwt(reshape(real(x),[m,n]), h, levels);
      [y2,l] = mdwt(reshape(imag(x),[m,n]), h, levels);
     y = y1 + sqrt(-1) * y2;    
   end   
   y = reshape(y,[m*n,1]);
end
