function p = simplex_proj(s,param)
%projection on the simplex being the intersection of the unit diamond and
%the positiv orthant.
%s : of size NxI with N: nb samples, I: nb dimensions
%p : the projection of s.
%param is optional
%param.algo = {'duchi','dykstra'}
%param.err for Dykstra only
%
%simon Arberet 08/02/2011
%
%   Url: http://unlocbox.sourceforge.net/rr/image_source_separation/misc/simplex_proj.php

% Copyright (C) 2013 LTS2-EPFL, by Nathanael Perraudin.
% This file is part of UNLocBoX-RR version 0.1.10
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

[N,I] = size(s);

if nargin<2
    param.algo = 'duchi';
    %algorithm of section 3 of John Duchi et al.
    %"Efficient Projections onto the ?1-Ball for Learning in High
    %Dimensions",ICML,2008.
end

if strcmp(param.algo,'dykstra')
    if isfield(param,'err')
      err = param.err;
    else
      err = 1e-6;
    end
    p = s;
    p_prev = p;
    %s1 = s;
    z = zeros(N,I);
    z1 =z;
    iter11 = 1;
    err11 = 1;
    
    %projection to convex constraint
    while (err11>err) && (iter11<100)
        
        %Dykstra's algorithm to project onto the intersection of convex
        %sets
        t = p - z1;
        s1 = t;
        s1(s1<0) = 0;
        z1 = s1 - t;
        
        t = s1 - z;
        SUM = sum(t,2);
        d = (1-SUM)/I; % L1 = 1
        p = t + repmat(d,1,I);
        z = p - t;
        
        err11 = norm(p_prev - p, 'fro')/norm(p_prev,'fro');
        p_prev = p;
        iter11 = iter11+1;
    end
    sprintf('Dykstra algorithm: %d iterations',iter11-1)
else %exact
    z=1;
    Mu = sort(s',1,'descend');
    Thetas = repmat((1./(1:I))',[1 N]).*(cumsum(Mu)-z);
    W = Mu - Thetas > 0;
    rho = sum(W);
    theta = Thetas(sub2ind([I,N],rho,1:N));
    w = max(s' - repmat(theta,[I 1]),0);
    p=w';
end